require(MASS)
require(nnet)

CVnn.cpus <- function(formula, data=cpus1[cpus.samp, ],
    size = c(0, 4, 4, 10, 10),
    lambda = c(0, rep(c(0.003, 0.01), 2)),
    nreps = 5, nifold = 10, ...)
{
  CVnn1 <- function(formula, data, nreps=1, ri,  ...)
  {
    truth <- log10(data$perf)
    res <- numeric(length(truth))
    cat("  fold")
    for (i in sort(unique(ri))) {
      cat(" ", i,  sep="")
      for(rep in 1:nreps) {
        learn <- nnet(formula, data[ri !=i,], trace=F, ...)
        res[ri == i] <- res[ri == i] +
                        predict(learn, data[ri == i,])
      }
    }
    cat("\n")
    sum((truth - res/nreps)^2)
  }
  choice <- numeric(length(lambda))
  ri <- sample(nifold, nrow(data), replace=T)
  for(j in seq(along=lambda)) {
    cat("  size =", size[j], "decay =", lambda[j], "\n")
    choice[j] <- CVnn1(formula, data, nreps=nreps, ri=ri,
                       size=size[j], decay=lambda[j], ...)
    }
  cbind(size=size, decay=lambda, fit=sqrt(choice/100))
}

predplot <- function(object, main="", len=100, ...)
{
   plot(Cushings[,1], Cushings[,2], log="xy", type="n",
     xlab="Tetrahydrocortisone", ylab = "Pregnanetriol", main=main)
   for(il in 1:4) {
     set <- Cushings$Type==levels(Cushings$Type)[il]
     text(Cushings[set, 1], Cushings[set, 2],
          labels=as.character(Cushings$Type[set]), col = 2 + il) }
   xp <- seq(0.6, 4.0, length=len)
   yp <- seq(-3.25, 2.45, length=len)
   cushT <- expand.grid(Tetrahydrocortisone=xp,
     Pregnanetriol=yp)
   Z <- predict(object, cushT, ...); zp <- as.numeric(Z$class)
   zp <- Z$post[,3] - pmax(Z$post[,2], Z$post[,1])
   contour(exp(xp), exp(yp), matrix(zp, len),
     add=T, levels=0, drawlabels=F)
   zp <- Z$post[,1] - pmax(Z$post[,2], Z$post[,3])
   contour(exp(xp), exp(yp), matrix(zp, len),
     add=T, levels=0, drawlabels=F)
   invisible()
}

class.ind <- function (cl)
{
    n <- length(cl)
    cl <- as.factor(cl)
    x <- matrix(0, n, length(levels(cl)))
    x[(1:n) + n * (unclass(cl) - 1)] <- 1
    dimnames(x) <- list(names(cl), levels(cl))
    x
}

data(Cushings, package="MASS")
Cushings <- get("Cushings", pos=1)
rm("Cushings", pos=1)
cush <- log(as.matrix(Cushings[, -3]))
tp <- factor(Cushings$Type[1:21])
Cf <- data.frame(tp = tp,
   Tetrahydrocortisone = log(Cushings[1:21,1]),
   Pregnanetriol = log(Cushings[1:21,2]) )
cush <- cush[1:21,]; tpi <- class.ind(tp)
rm(class.ind)

pltnn <- function(...)
{
  plot(Cushings[, 1], Cushings[, 2], log = "xy", type = "n", xlab =
       "Tetrahydrocortisone", ylab = "Pregnanetriol", ...)
  for(il in 1:4) {
    set <- Cushings$Type == levels(Cushings$Type)[il]
    text(Cushings[set, 1], Cushings[set, 2],
         as.character(Cushings$Type[set]), col = 2 + il)
  }
}

plt.bndry <- function(size = 0, decay = 0, ...)
{
  cush.nn <- nnet(cush, tpi, skip = T, softmax = T, size = size,
                  decay = decay, maxit = 1000)
  invisible(b1(predict(cush.nn, cushT), ...))
}

b1 <- function(Z, ...)
{
   zp <- Z[,3] - pmax(Z[,2], Z[,1])
   contour(exp(xp), exp(yp), matrix(zp, np),
      add=T, levels=0, drawlabels=F, ...)
   zp <- Z[,1] - pmax(Z[,3], Z[,2])
   contour(exp(xp), exp(yp), matrix(zp, np),
      add=T, levels=0, drawlabels=F, ...)
}

p0 <- function(x,y){
	d <- (x - 0.4)^2 + (y - 0.7)^2
	p <- exp(-d/f)
	d <- (x + 0.3)^2 + (y - 0.7)^2
	q <- exp(-d/f)
	(p+q)/(2*f*pi)
}
p1 <- function(x,y){
	d <- (x + 0.7)^2 + (y - 0.3)^2
	p <- exp(-d/f)
	d <- (x - 0.3)^2 + (y - 0.3)^2
	q <- exp(-d/f)
	(p+q)/(2*f*pi)
}
p0val <- matrix(0,57,66)
p1val <- matrix(0,57,66)
x <- seq(-1.24,1, 0.04)
y <- seq(-0.2,1.1,0.02)
f <- 0.06
for(i in 1:66) p0val[,i] <- p0(x, y[i])
for(i in 1:66) p1val[,i] <- p1(x, y[i])
rm(x,y,i,f)

synth.set <- function()
{
  x <- seq(-1.24,1, 0.04)
  y <- seq(-0.2,1.1,0.02)
  attach(synth.tr)
  eqscplot(range(x),range(y),type="n")
  contour(x,y,log(p1val)-log(p0val), add=T, levels=0, drawlabels=F, col=2)
  points(xs[1:125], ys[1:125], col=6,cex=0.5, pch=16)
  points(xs[126:250],ys[126:250],col=2,cex=0.5, pch=16)
  detach()
  invisible("Synth set up")
}


synth.fit <- function(col=4, skip=T, ...)
{
  fit <- nnet(yc ~ xs + ys, data=synth.tr, trace=F, skip=skip, ...)
  print(fit$value)
  XTest <- expand.grid(xs=seq(-1.24,1, 0.04), ys=seq(-0.2,1.1,0.02))
  pred <- predict(fit, XTest)
  prf <- predict(fit, synth.te)
  contour(seq(-1.24,1, 0.04), seq(-0.2,1.1,0.02),matrix(pred,57,66),
          add=T, levels=0.5, drawlabels=F, col=col)
  err <- sum((prf > 0.5) != (synth.te$yc == 1))/10
  cat("\nError rate:", err,"%\n\n")
  invisible()
}

synth.aver <- function(naver=25, col=1, skip=T, ...)
{
  pred <- matrix(0, 57, 66)
  XTest <- expand.grid(xs=seq(-1.24,1, 0.04), ys=seq(-0.2,1.1,0.02))
  for(i in 1:naver) {
	 cat(" ", i)
    fit <- nnet(yc ~ xs + ys, data=synth.tr, trace=F, skip=skip, ...)
    pred1 <- predict(fit, XTest)
    pred <- pred + matrix(pred1,57,66)
    contour(seq(-1.24,1, 0.04), seq(-0.2,1.1,0.02), matrix(pred1,57,66),
            add=T, levels=0.5, drawlabels=F, col=3)
  }
  contour(seq(-1.24,1, 0.04), seq(-0.2,1.1,0.02), pred, add=T,
          levels=0.5*naver, drawlabels=F, col=col, lwd=3)
  invisible()
}


knn1d <- function(train, test, cl)
{
  train <- as.matrix(train)
  if(is.null(dim(test))) dim(test) <- c(1, length(test))
  test <- as.matrix(test)
  p <- ncol(train)
  ntr <- nrow(train)
  if(length(cl) != ntr) stop("train and class have different lengths")
  nte <- nrow(test)
  if(ncol(test) != p) stop("Dims of test and train differ")
  clf <- as.factor(cl)
  nc <- max(unclass(clf))
  .C("VR_knn1",
     as.integer(ntr), as.integer(nte), as.integer(p), as.double(train),
     as.integer(unclass(clf)), as.double(test), res = integer(nte),
     integer(nc + 1), as.integer(nc), d = double(nte))$d
}
